/*****************************************************************************\
* (c) Copyright 2000-2019 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
#pragma once

// Include files
#include "AlignKernel/AlEquations.h"
#include "GaudiKernel/DataObject.h"
#include <ostream>

// Forward declarations

namespace LHCb {

  // Forward declarations

  // Class ID definition
  static const CLID CLID_AlignSummaryData = 10213;

  // Namespace for locations in TDS
  namespace AlignSummaryDataLocation {
    inline const std::string Default = "FST/Alignment/SummaryData";
  }

  /** @class AlignSummaryData AlignSummaryData.h
   *
   *
   *
   * @author
   *
   */

  class AlignSummaryData : public DataObject {
  public:
    /// constructor from AlEquations object
    AlignSummaryData( const Al::Equations& eq ) : m_equations( eq ) {}

    /// constructor
    AlignSummaryData( const size_t& size, const Gaudi::Time& time ) : m_equations( size, time ) {}

    /// Default Constructor
    AlignSummaryData() : m_equations() {}

    /// Default Destructor
    virtual ~AlignSummaryData() {}

    // Retrieve pointer to class definition structure
    const CLID&        clID() const override;
    static const CLID& classID();

    ///
    void add( const AlignSummaryData& rhs );

    /// needs also function with capital because GaudyPthon.Parallel uses that
    void Add( const AlignSummaryData& rhs );

    /// printOut method to Gaudi message stream
    virtual std::ostream& fillStream( std::ostream& os ) const override;

    /// Retrieve const
    const Al::Equations& equations() const;

    /// Retrieve
    Al::Equations& equations();

    /// Update
    void setEquations( const Al::Equations& value );

    friend std::ostream& operator<<( std::ostream& str, const AlignSummaryData& obj ) { return obj.fillStream( str ); }

  protected:
  private:
    Al::Equations m_equations; ///<

  }; // class AlignSummaryData

} // namespace LHCb

// -----------------------------------------------------------------------------
// end of class
// -----------------------------------------------------------------------------

// Including forward declarations

inline const CLID& LHCb::AlignSummaryData::clID() const { return LHCb::AlignSummaryData::classID(); }

inline const CLID& LHCb::AlignSummaryData::classID() { return CLID_AlignSummaryData; }

inline const Al::Equations& LHCb::AlignSummaryData::equations() const { return m_equations; }

inline Al::Equations& LHCb::AlignSummaryData::equations() { return m_equations; }

inline void LHCb::AlignSummaryData::setEquations( const Al::Equations& value ) { m_equations = value; }

inline void LHCb::AlignSummaryData::Add( const AlignSummaryData& rhs ) { add( rhs ); }
