/*****************************************************************************\
* (c) Copyright 2000-2018 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
#ifndef DToKPiTwoProng_H_
#define DToKPiTwoProng_H_

// from Gaudi
#include "Event/Track.h"
#include "GaudiAlg/GaudiTupleAlg.h"

namespace LHCb {

  class Particle;
  class TwoProngVertex;

} // namespace LHCb

class ILHCbMagnetSvc;
class IHitExpectation;
struct ITrackVertexer;
struct ITrackExtrapolator;
struct ITrackFitter;

class DToKPiTwoProng final : public GaudiTupleAlg {

public:
  DToKPiTwoProng( const std::string& name, ISvcLocator* pSvc );

  StatusCode initialize() override;

  StatusCode execute() override;

private:
  LHCb::Track* track( const LHCb::Particle* part ) const;

  std::unique_ptr<LHCb::TwoProngVertex> refittedMass( const LHCb::Track& track1, const LHCb::Track& track2,
                                                      double zVert ) const;

  double PIDK( const LHCb::Particle* part ) const;

  ITrackVertexer*     m_vertexer;
  ITrackExtrapolator* m_trackExtrapolator;

  ITrackFitter* m_trackFit;
  ITrackFitter* m_trackPreFit;

  std::string m_particleLocation;
  std::string m_vertexLocation;
  std::string m_trackOutputLocation;
  std::string m_resonanceName;

  double m_pionMass;
  double m_kaonMass;
  double m_deltaMass;
  double m_minMass;
  double m_maxMass;
};

#endif
