/*****************************************************************************\
* (c) Copyright 2000-2018 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
#ifndef _MUONTRACKSELECTOR_H_
#define _MUONTRACKSELECTOR_H_

// Include files
// from STD
#include <map>
#include <memory>
#include <string>

// from Gaudi
#include "GaudiAlg/GaudiAlgorithm.h"

// from Kernel
#include "Kernel/LHCbID.h"

// from TrackEvent
#include "Event/Track.h"

/** @class TrackFilterAlg TrackFilterAlg.h
 *  A small algorithm that filters tracks of a certain type.
 *
 *  @author Jan Amoraal
 *  @date   2007-07-11
 */
class MuonTrackSelector : public GaudiAlgorithm {

public:
  /// Some handy typedefs
  typedef std::vector<LHCb::LHCbID>                               LHCBIDS;
  typedef std::map<std::string, bool ( LHCb::LHCbID::* )() const> LHCbDetChecks;

  /// Standard constructor
  MuonTrackSelector( const std::string& name, ISvcLocator* pSvcLocator );

  StatusCode initialize() override; ///< Algorithm initialization
  StatusCode execute() override;    ///< Algorithm execution

private:
  bool printDebug() const { return msgLevel( MSG::DEBUG ); };

  void filterTracks( const LHCb::Tracks* tracks );

  void filterMuonTrack( LHCb::Track* track, LHCb::Tracks* outputContainer );

  std::string m_tracksInputContainer;  ///< Tracks input container
  std::string m_tracksOutputContainer; ///< Tracks output container
  std::string m_trackType;             ///< Type of tracks to filter
  std::string m_trackSelectorName;     ///< Name of track selector for alignment
  bool        m_calo;
  bool        m_noOverlap;
  double      m_pcut;
  double      m_muonChisquareCut;
  int         m_nStation;
  int         m_theR;
};

/*
#include "GaudiKernel/SystemOfUnits.h"
#include "TrackInterfaces/ITrackSelector.h"
#include <string>

class MuonTrackSelector : public ITrackSelector::TrackSelector {
    public:
        using TrackSelector::TrackSelector;
        bool accept( const LHCb::Track& aTrack ) const override;
    private:
        Gaudi::Property<double> m_pcut{this, "muonPcut", 0.*GeV};
        Gaudi::Property<double> m_muonChisquareCut{this, "muonChisquareCut", 0.};
        Gaudi::Property<bool> m_noOverlap{this, "noOverlap", false};
        Gaudi::Property<bool> m_Calo{this, "inCaloAcceptance", false};
        Gaudi::Property<int> m_nStation{this, "minHitStation", 2};
        Gaudi::Property<int> m_theR{this, "theRegion", -1};
};
*/

#endif
