/*****************************************************************************\
* (c) Copyright 2000-2018 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
// $Id: TrackMuonMatching.h,v 1.1 2010-06-04 13:02:57 svecchi Exp $
#ifndef TRACKMUONMATCHING_H
#define TRACKMUONMATCHING_H 1

// Include files
// from STD
#include <string>
#include <vector>

// from Gaudi
#include "GaudiAlg/GaudiTupleAlg.h"

// from TrackEvent
#include "Event/State.h"

// from TrackInterfaces
#include "TrackInterfaces/ITrackExtrapolator.h"

// from TrackTools
#include "TrackInterfaces/ITrackChi2Calculator.h"

#include "Event/Track.h"
#include "MuonDet/DeMuonDetector.h"

/** @class TrackMuonMatching TrackMuonMatching.h
 *
 *
 *  @author Stefania Vecchi
 *  @date   2010-06-04
 */

class TrackMuonMatching : public GaudiTupleAlg {

public:
  /// Standard constructor
  TrackMuonMatching( const std::string& name, ISvcLocator* pSvcLocator );

  virtual ~TrackMuonMatching(); ///< Destructor

  StatusCode initialize() override; ///< Algorithm initialization
  StatusCode execute() override;    ///< Algorithm execution
  // virtual StatusCode finalize  ();    ///< Algorithm finalization

protected:
private:
  StatusCode matchChi2( LHCb::State& tTrack, LHCb::State& mTrack, const double& atZ, double& chi2 );
  StatusCode OTmuonExtrap( LHCb::State* tState, const double& atZ );

private:
  ITrackExtrapolator*   m_extrapolator;
  std::string           m_nExtrapolator;
  ITrackChi2Calculator* m_chi2Calculator;
  std::string           m_nChi2Calculator;
  std::string           m_tTracksLocation;
  std::string           m_mTracksLocation;
  std::string           m_tracksOutputLocation;
  double                m_matchAtZ;
  bool                  m_matchAtFirstMuonHit;
  double                m_matchChi2Cut;
  bool                  m_allCombinations;
  DeMuonDetector*       m_muonDet;
  bool                  m_ntuple;

  static const int nSTATIONS = 4;
  double           m_zMS[nSTATIONS];
};

#endif // TRACKMUONMATCHING_H
