/*****************************************************************************\
* (c) Copyright 2000-2018 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
#include "AlignmentInterfaces/IWriteAlignmentConditionsTool.h"
#include "GaudiAlg/GaudiAlgorithm.h"
#include "GaudiKernel/ToolHandle.h"

/** @class WriteAlignmentConditions WriteAlignmentConditions.h
 *
 *  Class for printing out the full geometry tree
 *
 *  @author M.Needham
 *  @author J. van Tilburg
 *  @date   21/12/2006
 */

class WriteAlignmentConditions : public GaudiAlgorithm {

public:
  /// constructer
  WriteAlignmentConditions( const std::string& name, ISvcLocator* svcloc );

  /// destructer
  ~WriteAlignmentConditions();

  /// initialize
  StatusCode initialize() override;

  // finalize
  StatusCode finalize() override;

  /// empty execute
  StatusCode execute() override { return StatusCode::SUCCESS; }

private:
  ToolHandle<IWriteAlignmentConditionsTool> m_tool;
};

DECLARE_COMPONENT( WriteAlignmentConditions )

//--------------------------------------------------------------------
//
//  WriteAlignmentConditions : Prints out the full geometry tree
//
//--------------------------------------------------------------------

WriteAlignmentConditions::WriteAlignmentConditions( const std::string& name, ISvcLocator* pSvcLocator )
    : GaudiAlgorithm( name, pSvcLocator ), m_tool( "WriteAlignmentConditionsTool", this ) {
  // constructer
  declareProperty( "Tool", m_tool );
}

WriteAlignmentConditions::~WriteAlignmentConditions() {
  // destructer
}

StatusCode WriteAlignmentConditions::initialize() {
  StatusCode sc = GaudiAlgorithm::initialize();
  sc            = m_tool.retrieve();
  return sc;
}

StatusCode WriteAlignmentConditions::finalize() {
  StatusCode sc = m_tool->write();
  m_tool.release().ignore();
  if ( sc.isSuccess() ) sc = GaudiAlgorithm::finalize();
  return sc;
}
