#!/usr/bin/env bash
###############################################################################
# (c) Copyright 2000-2018 CERN for the benefit of the LHCb Collaboration      #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################

export 'firstRun'='191811' 
export 'begin'='24/05/2017' 
export 'lastRun'='500000' 
export 'today'=$(date +"%d/%m/%Y")
export 'outputFolder'='veloPlots'

while getopts "o:" opt; do
    case "$opt" in
    o)  outputFolder=$OPTARG
        ;;
    esac
done

rm -r $outputFolder
mkdir tmp

# Velo Tx Ty
$ALIGNMENTMONITORINGROOT/scripts/trendPlots.py -xsul  -a VeloLeft.Tx VeloLeft.Ty \
		-r $firstRun $lastRun \
		--labels VeloLeft.Tx:x-translation VeloLeft.Ty:y-translation --legTextSize 0.05 \
		--legTransparent --legTextSize 0.05 --legPosition .65 0.8 .9 .9 \
		--stickers "{(.2,.7,0.45,0.9, 0.05): ['#scale[1.5]{LHCb VELO}', '#scale[1.3]{Preliminary}'], (.65,.2,0.9,0.25,0.05): '$begin - $today',  (.25,.2,0.4,0.25,0.05): 'Empty markers = no update'}" \
		-o tmp/Velo_stability_Txy

$ALIGNMENTMONITORINGROOT/scripts/trendPlots.py -xsl  -a VeloLeft.Tx VeloLeft.Ty \
		-r $firstRun $lastRun \
		--labels VeloLeft.Tx:x-translation VeloLeft.Ty:y-translation --legTextSize 0.05 \
		--legTransparent --legTextSize 0.05 --legPosition .65 0.8 .9 .9 \
		--stickers "{(.2,.7,0.45,0.9, 0.05): ['#scale[1.5]{LHCb VELO}', '#scale[1.3]{Preliminary}'], (.65,.2,0.9,0.25,0.05): '$begin - $today',  (.25,.2,0.4,0.25,0.05): 'Empty markers = no update'}" \
		-o tmp/Velo_trend_Txy

$ALIGNMENTMONITORINGROOT/scripts/trendPlots.py -xsul  -a VeloLeft.Rx VeloLeft.Ry \
		-r $firstRun $lastRun \
		--labels VeloLeft.Rx:x-rotation VeloLeft.Ry:y-rotation --legTextSize 0.05 \
		--legTransparent --legTextSize 0.05 --legPosition .65 0.8 .9 .9 \
		--stickers "{(.2,.7,0.45,0.9, 0.05): ['#scale[1.5]{LHCb VELO}', '#scale[1.3]{Preliminary}'], (.65,.2,0.9,0.25,0.05): '$begin - $today',  (.25,.2,0.4,0.25,0.05): 'Empty markers = no update'}" \
		-o tmp/Velo_stability_Rxy

$ALIGNMENTMONITORINGROOT/scripts/trendPlots.py -xsl  -a VeloLeft.Rx VeloLeft.Ry \
		-r $firstRun $lastRun \
		--labels VeloLeft.Rx:x-rotation VeloLeft.Ry:y-rotation --legTextSize 0.05 \
		--legTransparent --legTextSize 0.05 --legPosition .65 0.8 .9 .9 \
		--stickers "{(.2,.7,0.45,0.9, 0.05): ['#scale[1.5]{LHCb VELO}', '#scale[1.3]{Preliminary}'], (.65,.2,0.9,0.25,0.05): '$begin - $today',  (.25,.2,0.4,0.25,0.05): 'Empty markers = no update'}" \
		-o tmp/Velo_trend_Rxy

$ALIGNMENTMONITORINGROOT/scripts/trendPlots.py -xsuld  -a VeloLeft.Tz \
		-r $firstRun $lastRun \
		--labels VeloLeft.Tz:z-translation  \
		--legTransparent --legTextSize 0.05 --legPosition .65 0.8 .9 .9 \
		--stickers "{(.2,.7,0.45,0.9, 0.05): ['#scale[1.5]{LHCb VELO}', '#scale[1.3]{Preliminary}'], (.65,.2,0.9,0.25,0.05): '$begin - $today',  (.25,.2,0.4,0.25,0.05): 'Empty markers = no update'}" \
		-o tmp/Velo_stability_Tz

$ALIGNMENTMONITORINGROOT/scripts/trendPlots.py -xsld  -a VeloLeft.Tz \
		-r $firstRun $lastRun \
		--labels VeloLeft.Tz:z-translation  \
		--legTransparent --legTextSize 0.05 --legPosition .65 0.8 .9 .9 \
		--stickers "{(.2,.7,0.45,0.9, 0.05): ['#scale[1.5]{LHCb VELO}', '#scale[1.3]{Preliminary}'], (.65,.2,0.9,0.25,0.05): '$begin - $today',  (.25,.2,0.4,0.25,0.05): 'Empty markers = no update'}" \
		-o tmp/Velo_trend_Tz

$ALIGNMENTMONITORINGROOT/scripts/trendPlots.py -xsuld  -a VeloLeft.Rz \
		-r $firstRun $lastRun \
		--labels VeloLeft.Rz:z-rotation  \
		--legTransparent --legTextSize 0.05 --legPosition .65 0.8 .9 .9 \
		--stickers "{(.2,.7,0.45,0.9, 0.05): ['#scale[1.5]{LHCb VELO}', '#scale[1.3]{Preliminary}'], (.65,.2,0.9,0.25,0.05): '$begin - $today',  (.25,.2,0.4,0.25,0.05): 'Empty markers = no update'}" \
		-o tmp/Velo_stability_Rz

$ALIGNMENTMONITORINGROOT/scripts/trendPlots.py -xsld  -a VeloLeft.Rz \
		-r $firstRun $lastRun \
		--labels VeloLeft.Rz:z-rotation  \
		--legTransparent --legTextSize 0.05 --legPosition .65 0.8 .9 .9 \
		--stickers "{(.2,.7,0.45,0.9, 0.05): ['#scale[1.5]{LHCb VELO}', '#scale[1.3]{Preliminary}'], (.65,.2,0.9,0.25,0.05): '$begin - $today',  (.25,.2,0.4,0.25,0.05): 'Empty markers = no update'}" \
		-o tmp/Velo_trend_Rz


mkdir $outputFolder
cp tmp/*.pdf $outputFolder
rm -r tmp
