/*****************************************************************************\
* (c) Copyright 2000-2018 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
// $Id: IGetElementsToBeAligned.h,v 1.11 2010-01-25 16:17:20 wouter Exp $
#ifndef IGETELEMENTSTOBEALIGNED_H
#define IGETELEMENTSTOBEALIGNED_H 1

// Include files
// from STL
#include <string>
#include <utility>
#include <vector>

// from Gaudi
#include "GaudiKernel/IAlgTool.h"

// from Kernel
#include "Kernel/LHCbID.h"

// from LoKi
#include "LoKi/Range.h"

/// Local
#include "AlignmentElement.h"

/** @class IGetElementsToBeAligned IGetElementsToBeAligned.h
 *
 *
 *  @author Jan Amoraal
 *  @date   2007-10-08
 */

namespace LHCb {
  class Measurement;
  class FitNode;
} // namespace LHCb

namespace Gaudi {
  class Time;
}

namespace Al {
  class Equations;
}

class IGetElementsToBeAligned : public extend_interfaces<IAlgTool> {

public:
  typedef std::vector<const AlignmentElement*> Elements;

  // Return the interface ID
  DeclareInterfaceID( IGetElementsToBeAligned, 2, 0 );

  // Return pair of forward begin iter and forward end iter
  virtual const Elements& elements() const = 0;

  // Return method that finds an alignment element for a given LHCb id
  virtual const AlignmentElement* findElement( const LHCb::LHCbID& anLHCbID ) const = 0;

  // Return method that finds an alignment element for a given Measuerment
  virtual const AlignmentElement* findElement( const LHCb::FitNode& node ) const = 0;

  // Return method that finds an alignment element for a given Measuerment
  virtual const AlignmentElement* findElement( const LHCb::Measurement& meas ) const = 0;

  // Return method that finds an alignment element for a given detector element
  virtual const AlignmentElement* findElement( const DetectorElement& element ) const = 0;

  // Find the list of elements corresponding to a path (which can ba rehulare expression)
  virtual StatusCode findElements( const std::string&                    path,
                                   std::vector<const AlignmentElement*>& alignelements ) const = 0;

  // return the time with which the geometry was initializes
  virtual Gaudi::Time initTime() const = 0;

  // initialize an Al::Equations object with the current par set
  virtual void initEquations( Al::Equations& ) const = 0;

  // initialize an Al::Equations object with the current par set
  virtual StatusCode initAlignmentFrame( Gaudi::Time now = 0 ) = 0;
};
#endif // IGETELEMENTSTOBEALIGNED_H
