/*****************************************************************************\
* (c) Copyright 2000-2018 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
/*
 * =====================================================================================
 *
 *       Filename:  Pi0CalibrationMonitor.h
 *
 *    Description:  main algorithm for pi0 calibration at LHCb online
 *
 *        Version:  1.0
 *        Created:  11/21/2016 01:47:23 PM
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Zhirui Xu (CNRS-LAPP), Zhirui.Xu@cern.ch
 *   Organization:
 *
 * =====================================================================================
 */

#ifndef CALIBRATION_PI0CALIBRATION_PI0CALIBRATIONMONITOR_H
#define CALIBRATION_PI0CALIBRATION_PI0CALIBRATIONMONITOR_H

#include <map>
#include <string>
#include <utility>
#include <vector>

#include "GaudiAlg/GaudiAlgorithm.h"
#include "GaudiKernel/Point3DTypes.h"
#include "Kernel/CaloCellID.h"

#include "Math/SMatrix.h"
#include "TH1.h"
#include "TH2.h"
#include "TTree.h"

#include "Pi0Calibration/Pi0LambdaMap.h"

class Pi0CalibrationMonitor : public GaudiAlgorithm {

public:
  Pi0CalibrationMonitor( const std::string& name, ISvcLocator* pSvcLocator );
  virtual ~Pi0CalibrationMonitor();

  virtual StatusCode initialize() override;
  virtual StatusCode execute() override;
  virtual StatusCode finalize() override;

  void plots();

private:
  std::string m_tupleFileName;
  std::string m_tupleName;
  std::string m_outputDir;
  std::string m_inputDir;

  double m_pdgPi0;

  std::vector<unsigned int>     m_indices;
  std::vector<LHCb::CaloCellID> m_cells;

  std::map<std::string, std::map<unsigned int, Gaudi::XYZPoint>> m_map;
  std::map<std::string, double>                                  m_cellsize;

  std::vector<LHCb::CaloCellID> m_skippedCells;
  void                          readCells( std::vector<LHCb::CaloCellID>& cells );
  void readValues( std::string varname, std::map<unsigned int, std::pair<double, double>>& values );
  void monitorPlots( std::string name, double centreVal, const std::map<unsigned int, double>& values );
  void monitorPlots( std::string name, double centreVal,
                     const std::map<unsigned int, std::pair<double, double>>& values );
};

#endif
